/***********************************************************
 * mult3.m
 *   Refer to mult3.c for documentation.
 %
 * Author: David Ross, Oct. 2005
 * $Id: mult3.c,v 1.3 2006/01/10 20:10:20 dross Exp $
 ***********************************************************/

#include "assert.h"
#include "stdio.h"
#include "mex.h"

void multiply(double *result, double *A, double *B, int I, int J, int K) ;

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]) {
   /* Declare variables. */ 
   double *tensor1 = NULL;
   double *tensor2 = NULL;
   const int *size = NULL;
   double *result = NULL;
   int result_size[3];
   int rows1, rows2, cols1, cols2, size_third_dim, n;

   /* Check for proper number of input and output arguments. */    
   if (nrhs != 2) {
      mexErrMsgTxt("Two input arguments required.");
   } 
   if (nlhs > 1) {
      mexErrMsgTxt("Too many output arguments.");
   }

   /* Check data type of input arguments. */
   if (!(mxIsDouble(prhs[0])) || !(mxIsDouble(prhs[1]))) {
      mexErrMsgTxt("Input arrays must be of type double.");
   }

   /* trivial case, if x and y are both 2 dimensional,
      then mult3 defaults to matrix multiplication */
   if (mxGetNumberOfDimensions(prhs[0]) == 2 && 
       mxGetNumberOfDimensions(prhs[1]) == 2) {
      mexCallMATLAB(1, plhs, 2, prhs, "mtimes");
      return;
   }

   /* Make sure the arguments have the correct dimensions */
   if (mxGetNumberOfDimensions(prhs[0]) != 3) {
      mexErrMsgTxt("First argument must be a 3-dimensional array");
   }
   if (mxGetNumberOfDimensions(prhs[1]) != 3) {
      mexErrMsgTxt("Second argument must be a 3-dimensional array");
   }

   /* Get the data. */
   tensor1 = (double *)mxGetPr(prhs[0]);
   tensor2 = (double *)mxGetPr(prhs[1]);
   size = mxGetDimensions(prhs[0]);
   rows1 = size[0];
   cols1 = size[1];
   size_third_dim = size[2];
   size = mxGetDimensions(prhs[1]);
   rows2 = size[0];
   cols2 = size[1];
   
   if (cols1 != rows2) {
      mexErrMsgTxt("size(x,2) and size(y,1) must match for matrix multiplication");
   }

   if (size_third_dim != size[2]) {
      mexErrMsgTxt("size of 3rd dimension of both matrices must match");
   }
   
   /* Allocate the output variable */
   result_size[0] = rows1;
   result_size[1] = cols2;
   result_size[2] = size_third_dim;
   plhs[0] = mxCreateNumericArray(3, result_size, mxDOUBLE_CLASS, mxREAL);
   result = mxGetPr(plhs[0]);

   /* do the multiplication */
   for(n = 0;  n < size_third_dim; n++) {
      multiply(result+(rows1*cols2*n), tensor1+(rows1*cols1*n), 
	       tensor2+(rows2*cols2*n), rows1, cols1, cols2);
   }
}

void multiply(double *result, double *A, double *B, int I, int J, int K) {
   int i,j,k;
   for(i=0; i < I; i++) {
      for(k=0; k < K; k++) {
	 result[I*k+i] = 0.0;
	 for(j=0; j < J; j++) {
	    /* result(i,k) += A(i,j) * B(j,k) */
	    result[I*k+i] += A[I*j+i] * B[J*k+j];
	 }
      }
   }
}
