/***********************************************************
 * sample_discrete2.c
 *   Refer to sample_discrete2.m for documentation.
 * 
 * Author: David Ross, 2004.
 * $Id$
 ***********************************************************/

#include "mex.h"
#include <math.h>
#include <stdlib.h>

void sample_discrete(double *prob, int rows, int cols, double *samples,
		     int num_samples);

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, 
		 const mxArray *prhs[]) {
   int cols, rows, num_samples;
   double *probability, *samples;

   /* check for the proper number of arguments */
   if (nrhs != 2) {
      mexErrMsgTxt("wrong number of input arguments");
   }
   if (nlhs != 1) {
      mexErrMsgTxt("wrong number of output arguments");
   }

   /* get size of probability */
   rows = mxGetM(prhs[0]);
   cols = mxGetN(prhs[0]);

   /* do some simple checks on probability */
   if (!mxIsDouble(prhs[0]) || cols < 1 || rows < 1) {
      mexErrMsgTxt("probability has invalid size");
   }

   /* get probability */
   probability = mxGetPr(prhs[0]);


   /* make sure the # of samples to get is okay */
   if (!mxIsDouble(prhs[1]) || mxGetM(prhs[1]) != 1 || 
       mxGetN(prhs[1]) != 1) {
       mexErrMsgTxt("num_samples must be a scalar");
   }

   /* get the number of samples */
   num_samples = (int)mxGetScalar(prhs[1]);
   
   if (num_samples < 1) {
      mexErrMsgTxt("num_samples must be at least 1");
   }
   

   /* create the result */
   plhs[0] = mxCreateDoubleMatrix(num_samples, cols, mxREAL);
   samples = mxGetPr(plhs[0]);
   
   /* do it */
   sample_discrete(probability, rows, cols, samples, num_samples);
}


void sample_discrete(double *prob, int rows, int cols, double *samples, 
		     int num_samples) {
   int n=0,b=0,c=0;
   double urand = 0.0;
   double *cumsum = NULL;

   cumsum = mxMalloc(sizeof(double)*rows);
   
   for(c=0; c < cols; c++) {

      cumsum[0] = prob[c*rows];
      for(b=1; b < rows; b++) {
	 cumsum[b] = cumsum[b-1] + prob[c*rows+b];
      }
    
      for(n=0; n < num_samples; n++) {
	 urand = ((double)rand()) / RAND_MAX;
	 for(b=0; b < rows-1 && urand >= cumsum[b]; b++);
	 *(samples++) = (double)(b+1);
      }
   }
   mxFree(cumsum);

}
