/***********************************************************
 * simple_histeq.c
 *   Refer to simple_histeq.m for documentation.
 *
 * Author: David Ross, Nov. 22, 2003.
 * $Id: simple_histeq.c,v 1.1 2004/02/11 18:44:44 dross Exp $
 ***********************************************************/

#include "mex.h"
#include <math.h>

void simple_histeq(double *image, int size, double *result);

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
   int Nx, Ny;
   double *image, *result;

   /* check for the proper number of arguments */
   if (nrhs != 1) {
      mexErrMsgTxt("wrong number of input arguments");
   }
   if (nlhs != 1) {
      mexErrMsgTxt("wrong number of output arguments");
   }

   /* get sizes of x and y*/
   Ny = mxGetM(prhs[0]);
   Nx = mxGetN(prhs[0]);

   /* make sure x and y are okay */
   if (!mxIsDouble(prhs[0]) || Nx < 1 || Ny < 1) {
      mexErrMsgTxt("x and/or y are invalid");
   }

   /* get x and y */
   image = mxGetPr(prhs[0]);

   /* create the result */
   plhs[0] = mxCreateDoubleMatrix(Ny, Nx, mxREAL);
   result = mxGetPr(plhs[0]);

   /* do it */
   simple_histeq(image, Nx*Ny, result);
}


void simple_histeq(double *image, int size, double *result) {
   int i;
   register int val;
   int counts[256];

   /* initalize the counts to zero */
   for(i=0; i < 256; i++) {
      counts[i] = 0;
   }

   /* create the histogram */
   for(i=0; i < size; i++) {
      val = (int) image[i];
      /*if (val < 0 || val > 255) {
	 mexErrMsgTxt("invalid pixel value");
	 return;
	 }*/
      counts[val] += 1;
   }

   /* make the counts into a cumulative sum */
   for(i=1; i < 256; i++) {
      counts[i] += counts[i-1];
   }

   /* create the output image, using the pixel value map */
   for(i=0; i < size; i++) {
      result[i] = ((double) counts[(int)image[i]]) / size;
   }
}
