% test_resample.m
% 
% Author: David Ross, 2006.
% $Id$

function test_resample()
    
% load an image
img = imread('basketball.jpg');

% specify the position/scale/orientation to sample from
x = 300;
y = 88;
theta = 45;
scale = 10;
T = affine_matrix_from_params([x y theta, scale]);

% dimensions of the new resampled window
width = 30;
height = 30;

[xx,yy] = meshgrid(linspace(-1,1,width),linspace(-1,1,height));
samples = T * [xx(:) yy(:) ones(width*height,1)]';

% Seriously consider bluring img first if width and height are
% much smaller than width and height.
window = resample(double(img), samples(1,:), samples(2,:));
window = reshape(window,[height width]);

figure(1); subplot(1,2,1);
imagesc(img);
axis image; colormap('gray');

subplot(1,2,2);
imagesc(window); 
axis image; colormap('gray');

end


function [A] = affine_matrix_from_params(params)
% [A] = affine_matrix_from_params(params)
%    Create an affine transformation matrix from the given parameters.
%
% Required Input:
%    params(1) = x translation (pixels)
%    params(2) = y translation (pixels)
%    params(3) = rotation angle (radians)
%    params(4) = x scaling factor (pixels)
%    params(5) = y scaling factor (pixels)
%    params(6) = scaling angle (radians)
%
%    Note: if elements 5 and 6 of params are omitted, then the
%    result is a similarity transform (i.e. scaling angle=0 and
%    y scaling = x scaling).
%
% Output:
%    A      = 3x3 transformation matrix

% Author: David Ross, Novemeber 16, 2003.
% $Id: affine_matrix_from_params.m,v 1.1 2004/02/11 18:50:01 dross Exp $

error(nargchk(1,1,nargin));

if length(params == 4)
    params(5) = params(4);
    params(6) = 0;
end

% rotation
A = [cos(params(3)) sin(params(3)); -sin(params(3)) cos(params(3))];

% inverse scaling rotation
A = A * [cos(-params(6)) sin(-params(6)); -sin(-params(6)) cos(-params(6))];

% scaling
A = A * [params(4) 0; 0 params(5)];

% scaling rotation
A = A * [cos(params(6)) sin(params(6)); -sin(params(6)) ...
	 cos(params(6))];

% incorporate translation
A = [A [params(1); params(2)]; 0 0 1];

end
